//
// Example of using OpenAL - check supported extensions & EFX
//
// Author: Alex V. Boreskoff <steps3d@narod.ru>, <alexboreskoff@mtu-net.ru>
//

#include	<stdlib.h>
#include	<stdio.h>
#include	<time.h>

#include	<al.h>
#include	<alc.h>
#include	<efx.h>

int main ( int argc, char  * argv [] )
{
	ALCdevice * device = alcOpenDevice ( NULL );

	if ( device == NULL )
	{
		printf ( "Error opening default device\n" );

		exit ( 1 );
	}
	const char * str = (const char *) alcGetString ( device, ALC_EXTENSIONS );

	if ( str != NULL )
		printf ( "Extensions:\n%s\n", str );
	else
	    printf ( "No extension found\n" );

	if ( !alcIsExtensionPresent ( device, "ALC_EXT_EFX" ) )
	{
		printf ( "No EFX.\n" );
		
		return 1;
	}
	
	int	attrs [4] = { ALC_MAX_AUXILIARY_SENDS, 4, 0, 0 };
	
	ALCcontext * context = alcCreateContext ( device, attrs );

	if ( context == NULL )
	{
		printf ( "Error creating context\n" );

		exit ( 1 );
	}

	int	maxSends, verMajor, verMinor;
	
	alcMakeContextCurrent ( context );

	alcGetIntegerv ( device, ALC_MAX_AUXILIARY_SENDS, 1, &maxSends );
	alcGetIntegerv ( device, ALC_EFX_MAJOR_VERSION,   1, &verMajor );
	alcGetIntegerv ( device, ALC_EFX_MAJOR_VERSION,   1, &verMinor );
	
	printf ( "maxSends: %d\nverMajor %d\nverMinor %d\n", maxSends, verMajor, verMinor );
	
	char * captureDevices = (char *) alcGetString ( NULL, ALC_CAPTURE_DEVICE_SPECIFIER );
		
	if ( captureDevices == NULL )
		printf ( "No capture devices found\n" );
	else
		printf ( "Capture devices: %s", captureDevices );
		
	alcMakeContextCurrent ( NULL    );
	alcDestroyContext     ( context );
	alcCloseDevice        ( device  );

	return 0;
}
